import logging
import os
import time
import multiprocessing
from concurrent.futures import ThreadPoolExecutor

logger = logging.getLogger(__name__)
deleter_lock = multiprocessing.Lock()

class Deleter:

    def __init__(self, log_dir: str, retention: int) -> None:
        self._executor = ThreadPoolExecutor(1)
        self._log_dir = log_dir
        self._retention = retention

    def __call__(self) -> None:
        if self._retention <= 0:
            return

        now = time.time()
        min_modified_time = now - self._retention

        with deleter_lock:
            try:
                for f in os.listdir(self._log_dir):
                    file_path = os.path.join(self._log_dir, f)
                    if not os.path.isfile(file_path):
                        continue

                    stats = os.stat(file_path)
                    last_modified = stats.st_mtime
                    if last_modified < min_modified_time:
                        os.remove(file_path)
            except (FileNotFoundError, IOError):
                logger.exception(
                    f"Failed to delete files modified before {min_modified_time}.")



import bz2
import gzip
import logging
import lzma
import multiprocessing
import os
from concurrent.futures import ThreadPoolExecutor

logger = logging.getLogger(__name__)
compressor_lock = multiprocessing.Lock()


class AsyncCompressor:

    def __init__(self, compression_format: str) -> None:
        if compression_format is not None:
            compression_format = compression_format.lower()
        if compression_format == 'gz' or compression_format == 'gzip':
            self._extension = 'gz'
            self._open_out = gzip.open
        elif compression_format == 'bz2' or compression_format == 'bzip2':
            self._extension = 'bz2'
            self._open_out = bz2.open
        elif compression_format == 'lzma':
            self._extension = 'lzma'
            self._open_out = lzma.open
        else:
            self._extension = None
            self._open_out = None
        self._executor = ThreadPoolExecutor(1)

    def __call__(self, source: str, dest: str = None) -> None:
        if self._open_out is None:
            return
        if not os.path.isfile(source):
            return
        if dest is None:
            if self._extension is None:
                raise ValueError('Unknown file extension')
            dest = f"{source}.{self._extension}"
        self._executor.submit(self._gz, source, dest)

    def _gz(self, source: str, dest: str) -> None:
        with compressor_lock:
            try:
                if os.path.isfile(source):
                    f_in = open(source, 'rb')
                    f_out = self._open_out(dest, 'wb')
                    f_out.writelines(f_in)
                    f_out.close()
                    f_in.close()
                    os.remove(source)
            except (FileNotFoundError, IOError):
                logger.exception(f"Failed to compress {source} into {dest}.")



import os
import time
from logging.handlers import TimedRotatingFileHandler
from typing import Optional

from contek_timbersaw.async_compressor import AsyncCompressor
from contek_timbersaw.deleter import Deleter


class TimedRollingFileHandler(TimedRotatingFileHandler):

    def __init__(
            self,
            log_dir: str,
            file_suffix: str = '.log',
            compression_format: Optional[str] = None,
            retention: int = 0,
            **kwargs,
    ) -> None:
        super().__init__(log_dir, delay=True, **kwargs)
        self._log_dir = log_dir
        self._file_suffix = file_suffix
        self._compress = AsyncCompressor(compression_format)
        self._delete = Deleter(log_dir, retention)
        self._delete()
        self._update_current_file()

    def doRollover(self):
        self.close()
        self._delete()
        self._update_current_file()
        self._calculate_new_rollover_at()

    def _calculate_new_rollover_at(self) -> None:
        current_time = int(time.time())
        new_rollover_at = self.computeRollover(current_time)
        while new_rollover_at <= current_time:
            new_rollover_at = new_rollover_at + self.interval

        if self._should_adjust_for_dst_change():
            dst_now = time.localtime(current_time)[-1]
            dst_at_rollover = time.localtime(new_rollover_at)[-1]
            if dst_now != dst_at_rollover:
                if not dst_now:  # DST kicks in before next rollover, so we need to deduct an hour
                    addend = -3600
                else:  # DST bows out before next rollover, so we need to add an hour
                    addend = 3600
                new_rollover_at += addend

        self.rolloverAt = new_rollover_at

    def _should_adjust_for_dst_change(self) -> bool:
        if self.utc:
            return False
        return self.when == 'MIDNIGHT' or self.when.startswith('W')

    def _update_current_file(self) -> None:
        if self.utc:
            time_tuple = time.gmtime()
        else:
            time_tuple = time.localtime()

        time_str = time.strftime(self.suffix, time_tuple)
        file_name = time_str + self._file_suffix
        new_file = os.path.join(self._log_dir, file_name)
        if self.baseFilename is not None:
            if new_file == self.baseFilename:
                return
            self._compress(self.baseFilename)
        self.baseFilename = new_file



import logging.config
import os
import sys
import time

from contek_timbersaw.timed_rolling_file_handler import TimedRollingFileHandler


def setup():
    log_format = os.getenv(
        'log_format',
        '%(asctime)s %(levelname)s %(filename)s:%(lineno)d - %(message)s',
    )
    log_date_format = os.getenv(
        'log_date_format',
        '%Y-%m-%dT%H:%M:%S',
    )
    log_root = os.getenv('log_root', os.path.join(os.getcwd(), 'logs'))
    log_rolling = os.getenv('log_rolling', 'MIDNIGHT')
    log_utc = bool(os.getenv('log_utc', False))
    log_info_retention_days = int(os.getenv('log_info_retention_days', '7'))
    log_warn_retention_days = int(os.getenv('log_warn_retention_days', '14'))
    log_error_retention_days = int(os.getenv('log_error_retention_days', '28'))

    logger = logging.getLogger()
    formatter = logging.Formatter(fmt=log_format, datefmt=log_date_format)
    if log_utc:
        formatter.converter = time.gmtime
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(formatter)
    stream_handler.setStream(sys.stdout)
    logger.addHandler(stream_handler)
    logger.setLevel(logging.INFO)
    logger.propagate = True

    def add_file_handler(level, retention_days, compression_format=None):
        file_dir = os.path.join(log_root, level.lower())
        os.makedirs(file_dir, exist_ok=True)
        handler = TimedRollingFileHandler(
            file_dir,
            compression_format=compression_format,
            retention=retention_days * 24 * 60 * 60,
            when=log_rolling,
            utc=log_utc,
        )
        handler.setFormatter(formatter)
        handler.setLevel(logging.getLevelNamesMapping()[level])
        logger.addHandler(handler)

    add_file_handler('INFO', log_info_retention_days, 'gz')
    add_file_handler('WARN', log_warn_retention_days)
    add_file_handler('ERROR', log_error_retention_days)

    def handle_exception(exc_type, exc_value, exc_traceback) -> None:
        if issubclass(exc_type, KeyboardInterrupt):
            sys.__excepthook__(exc_type, exc_value, exc_traceback)
            return

        logger.exception(
            "Unhandled exception.",
            exc_info=(exc_type, exc_value, exc_traceback),
        )

    sys.excepthook = handle_exception
